@@ -39,6 +39,7 @@
 #include "gnutls_kx.h"
 #include "gnutls_record.h"
 #include "gnutls_constate.h"
+#include <random.h>
 #include <gc.h>
 
 inline static int
@@ -193,29 +194,32 @@ _gnutls_decrypt (gnutls_session_t session, opaque * ciphertext,
   return ret;
 }
 
-inline static mac_hd_t
-mac_init (gnutls_mac_algorithm_t mac, opaque * secret, int secret_size,
+inline static int
+mac_init (digest_hd_st* td, gnutls_mac_algorithm_t mac, opaque * secret, int secret_size,
 	  int ver)
 {
-  mac_hd_t td;
+int ret =  0;
 
   if (mac == GNUTLS_MAC_NULL)
-    return GNUTLS_MAC_FAILED;
+    {
+      gnutls_assert();
+      return GNUTLS_E_HASH_FAILED;
+    }
 
   if (ver == GNUTLS_SSL3)
     {				/* SSL 3.0 */
-      td = _gnutls_mac_init_ssl3 (mac, secret, secret_size);
+      ret = _gnutls_mac_init_ssl3 (td, mac, secret, secret_size);
     }
   else
     {				/* TLS 1.x */
-      td = _gnutls_hmac_init (mac, secret, secret_size);
+      ret = _gnutls_hmac_init (td, mac, secret, secret_size);
     }
 
-  return td;
+  return ret;
 }
 
 inline static void
-mac_deinit (mac_hd_t td, opaque * res, int ver)
+mac_deinit (digest_hd_st *td, opaque * res, int ver)
 {
   if (ver == GNUTLS_SSL3)
     {				/* SSL 3.0 */
@@ -233,7 +237,7 @@ calc_enc_length (gnutls_session_t session, int data_size,
 		 cipher_type_t block_algo, uint16_t blocksize)
 {
   uint8_t rnd;
-  int length;
+  int length, ret;
 
   *pad = 0;
 
@@ -244,10 +248,11 @@ calc_enc_length (gnutls_session_t session, int data_size,
 
       break;
     case CIPHER_BLOCK:
-      if (gc_nonce (&rnd, 1) != GC_OK)
+      ret =_gnutls_rnd (RND_NONCE, &rnd, 1);
+      if ( ret < 0)
 	{
 	  gnutls_assert ();
-	  return GNUTLS_E_RANDOM_FAILED;
+	  return ret;
 	}
 
       /* make rnd a multiple of blocksize */
@@ -298,7 +303,7 @@ _gnutls_compressed2ciphertext (gnutls_session_t session,
   uint16_t c_length;
   uint8_t pad;
   int length, ret;
-  mac_hd_t td;
+  digest_hd_st td;
   uint8_t type = _type;
   uint8_t major, minor;
   int hash_size =
@@ -320,34 +325,33 @@ _gnutls_compressed2ciphertext (gnutls_session_t session,
 
 
   /* Initialize MAC */
-  td = mac_init (session->security_parameters.write_mac_algorithm,
+  ret = mac_init (&td, session->security_parameters.write_mac_algorithm,
 		 session->connection_state.write_mac_secret.data,
 		 session->connection_state.write_mac_secret.size, ver);
 
-  if (td == GNUTLS_MAC_FAILED
-      && session->security_parameters.write_mac_algorithm != GNUTLS_MAC_NULL)
+  if (ret < 0 && session->security_parameters.write_mac_algorithm != GNUTLS_MAC_NULL)
     {
       gnutls_assert ();
-      return GNUTLS_E_INTERNAL_ERROR;
+      return ret;
     }
 
   c_length = _gnutls_conv_uint16 (compressed.size);
 
-  if (td != GNUTLS_MAC_FAILED)
+  if (session->security_parameters.write_mac_algorithm != GNUTLS_MAC_NULL)
     {				/* actually when the algorithm in not the NULL one */
-      _gnutls_hmac (td,
+      _gnutls_hmac (&td,
 		    UINT64DATA (session->connection_state.
 				write_sequence_number), 8);
 
-      _gnutls_hmac (td, &type, 1);
+      _gnutls_hmac (&td, &type, 1);
       if (ver >= GNUTLS_TLS1)
 	{			/* TLS 1.0 or higher */
-	  _gnutls_hmac (td, &major, 1);
-	  _gnutls_hmac (td, &minor, 1);
+	  _gnutls_hmac (&td, &major, 1);
+	  _gnutls_hmac (&td, &minor, 1);
 	}
-      _gnutls_hmac (td, &c_length, 2);
-      _gnutls_hmac (td, compressed.data, compressed.size);
-      mac_deinit (td, MAC, ver);
+      _gnutls_hmac (&td, &c_length, 2);
+      _gnutls_hmac (&td, compressed.data, compressed.size);
+      mac_deinit (&td, MAC, ver);
     }
 
 
@@ -376,11 +380,13 @@ _gnutls_compressed2ciphertext (gnutls_session_t session,
     {
       /* copy the random IV.
        */
-      if (gc_nonce (data_ptr, blocksize) != GC_OK)
+      ret = _gnutls_rnd (RND_NONCE, data_ptr, blocksize);
+      if (ret < 0)
 	{
 	  gnutls_assert ();
-	  return GNUTLS_E_RANDOM_FAILED;
+	  return ret;
 	}
+
       data_ptr += blocksize;
     }
 
@@ -400,7 +406,7 @@ _gnutls_compressed2ciphertext (gnutls_session_t session,
 
   /* Actual encryption (inplace).
    */
-  ret = _gnutls_cipher_encrypt (session->connection_state.
+  ret = _gnutls_cipher_encrypt (&session->connection_state.
 				write_cipher_state, cipher_data, length);
   if (ret < 0)
     {
@@ -424,7 +430,7 @@ _gnutls_ciphertext2compressed (gnutls_session_t session,
   uint16_t c_length;
   uint8_t pad;
   int length;
-  mac_hd_t td;
+  digest_hd_st td;
   uint16_t blocksize;
   int ret, i, pad_failed = 0;
   uint8_t major, minor;
@@ -442,11 +448,11 @@ _gnutls_ciphertext2compressed (gnutls_session_t session,
 
   /* initialize MAC 
    */
-  td = mac_init (session->security_parameters.read_mac_algorithm,
+  ret = mac_init (&td, session->security_parameters.read_mac_algorithm,
 		 session->connection_state.read_mac_secret.data,
 		 session->connection_state.read_mac_secret.size, ver);
 
-  if (td == GNUTLS_MAC_FAILED
+  if (ret < 0
       && session->security_parameters.read_mac_algorithm != GNUTLS_MAC_NULL)
     {
       gnutls_assert ();
@@ -460,7 +466,7 @@ _gnutls_ciphertext2compressed (gnutls_session_t session,
 	  (session->security_parameters.read_bulk_cipher_algorithm))
     {
     case CIPHER_STREAM:
-      if ((ret = _gnutls_cipher_decrypt (session->connection_state.
+      if ((ret = _gnutls_cipher_decrypt (&session->connection_state.
 					 read_cipher_state,
 					 ciphertext.data,
 					 ciphertext.size)) < 0)
@@ -479,7 +485,7 @@ _gnutls_ciphertext2compressed (gnutls_session_t session,
 	  return GNUTLS_E_DECRYPTION_FAILED;
 	}
 
-      if ((ret = _gnutls_cipher_decrypt (session->connection_state.
+      if ((ret = _gnutls_cipher_decrypt (&session->connection_state.
 					 read_cipher_state,
 					 ciphertext.data,
 					 ciphertext.size)) < 0)
@@ -537,24 +543,24 @@ _gnutls_ciphertext2compressed (gnutls_session_t session,
   /* Pass the type, version, length and compressed through
    * MAC.
    */
-  if (td != GNUTLS_MAC_FAILED)
+  if (session->security_parameters.read_mac_algorithm != GNUTLS_MAC_NULL)
     {
-      _gnutls_hmac (td,
+      _gnutls_hmac (&td,
 		    UINT64DATA (session->connection_state.
 				read_sequence_number), 8);
 
-      _gnutls_hmac (td, &type, 1);
+      _gnutls_hmac (&td, &type, 1);
       if (ver >= GNUTLS_TLS1)
 	{			/* TLS 1.x */
-	  _gnutls_hmac (td, &major, 1);
-	  _gnutls_hmac (td, &minor, 1);
+	  _gnutls_hmac (&td, &major, 1);
+	  _gnutls_hmac (&td, &minor, 1);
 	}
-      _gnutls_hmac (td, &c_length, 2);
+      _gnutls_hmac (&td, &c_length, 2);
 
       if (length > 0)
-	_gnutls_hmac (td, ciphertext.data, length);
+	_gnutls_hmac (&td, ciphertext.data, length);
 
-      mac_deinit (td, MAC, ver);
+      mac_deinit (&td, MAC, ver);
     }
 
   /* This one was introduced to avoid a timing attack against the TLS
